package org.litesoft.orsup.selection;

import org.litesoft.util.IllegalArgument;

// Copyright Status:
//
// All Software available from LiteSoft.org (including this file) is
// hereby released into the public domain.
//
// It is free!  As in, you may use it freely in both commercial and
// non-commercial applications, bundle it with your software
// distribution, include it on a CD-ROM, list the source code in a book,
// mirror the documentation at your own web site, or use it in any other
// way you see fit.
//
// NO Warranty!
//
// All software is provided "as is".
//
// There is ABSOLUTELY NO WARRANTY OF ANY KIND: not for the design, fitness
// (for a particular purpose), level of errors (or lack thereof), or
// applicability of this software.  The entire risk as to the quality
// and performance of this software is with you.  Should this software
// prove defective, you assume the cost of all necessary servicing, repair
// or correction.
//
// In no event unless required by applicable law or agreed to in writing
// will any party who created or may modify and/or redistribute this
// software, be liable to you for damages, including any general,
// special, incidental or consequential damages arising out of the use or
// inability to use this software (including but not limited to loss of
// data or data being rendered inaccurate or losses sustained by you or
// third parties or a failure of this software to operate with any
// other programs), even if such holder or other party has been advised
// of the possibility of such damages.
//
// NOTE: Should you discover a bug, have a recogmendation for a change, wish
// to submit modifications, or wish to add new classes/functionality,
// please email them to:
//
//        changes@litesoft.org
//

/**
 * An abstract representation of a wrapped SQL <i>WHERE</i> clause (like NOT).<p>
 *
 * The purpose of this class is to support the unary operations (like NOT)
 * against another <b>WhereClause</b>.<p>
 *
 * This form of <b>WhereClause</b> is currently used only to support:
 * NOT.<p>
 *
 * See <a href="WhereClause.html">WhereClause</a><br>
 * See <a href="WhereClauseFactory.html">WhereClauseFactory</a><p>
 *
 * Exceptions: All problems caught when the parameter(s) are checked (as
 * indicated/implied in the @param tags) will generate an IllegalArgumentException,
 * and means the API user has a problem.  If a NullPointerException (or some
 * others, like: ClassCastException or ArrayIndexOutOfBoundsException) is thrown,
 * it means the API developer has a problem.  Any Exception that is explicitly
 * thrown in the API, but unrelated to a parameter, will be listed in the throws
 * clause (and hopefully in the tag @throws).  These may (but probably won't) be
 * checked Exceptions.
 *
 * @author  George Smith
 * @version 1.03 02/07/02 Exception Policy, Use of IllegalArgument class.
 * @version 1.02 11/05/01 Completed? JavaDocs
 * @version 1.0 10/07/01
 */

public abstract class AbstractWhereClauseWrapper extends WhereClause
{
    private WhereClause zWrappedWhereClause;
    private String zWrapperToString;
    private String zWrapperToSql;

    /**
     * Constructor that supports a <i>wrapped</i> SQL <i>WHERE</i> clause.<p>
     *
     * @param   pType               The Type for this WhereClause.
     * @param   pWrapperToString    Unary Operator for toString() (!null).  eg "NOT"
     * @param   pWrapperToSql       Unary Operator for toSQL() (!null).     eg "NOT"
     * @param   pWrappedWhereClause The WhereClause that is being wrapped (!null).
     */
    protected AbstractWhereClauseWrapper( int pType ,
                                          String pWrapperToString ,
                                          String pWrapperToSql ,
                                          WhereClause pWrappedWhereClause )
    {
        super( pType );

        IllegalArgument.ifNull( "To String Wrapper" , zWrapperToString = pWrapperToString);
        IllegalArgument.ifNull( "To SQL Wrapper" , zWrapperToSql = pWrapperToSql);
        IllegalArgument.ifNull( "Wrapped WhereClause" , zWrappedWhereClause = pWrappedWhereClause);
    }

    /**
     * Accessor for the Wrapped WhereClause.<p>
     *
     * @return Wrapped WhereClause (!null).
     */
    public final WhereClause getWrappedWhereClause()
    {
        return zWrappedWhereClause;
    }

    /**
     * Helper method for <b>toString()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see WhereClause#toString()
     */
    protected final void toStringHelper( StringBuffer pSB )
    {
        toStringHelperParenthesizer( pSB , zWrapperToString , getWrappedWhereClause() );
    }

    /**
     * Helper method for <b>toSQL()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * Note: May need to override to get proper result for SQL.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see WhereClause#toSQL()
     */
    protected void toSqlHelper( StringBuffer pSB )
    {
        toSqlHelperParenthesizer( pSB , zWrapperToSql , getWrappedWhereClause() );
    }
}
